# Add a test of the sign matrix (function CalcContrastMatrix_SBP)

#' Check constant sum value
#'
#' @param constSumValue Positive-value scalar that is the sum of the
#' elements in the composition
#'
#' @noRd
CheckConstSumValue <- function(constSumValue){
  stopifnot(constSumValue > 0.0)
}

#' Check whether x is a valid unclosed, compositional vector
#'
#' @param x Unclosed compositional vector
#'
#' @noRd
CheckCompVector_Unclosed <- function(x){
  stopifnot(is.vector(x))
  stopifnot(!is.na(x))
  stopifnot(all(0 < x))
}

#' Check whether x is a valid closed, compositional vector
#'
#' @param x Closed compositional vector
#'
#' @param constSumValue Positive-value scalar that is the sum of the
#' elements in the composition
#'
#' @noRd
CheckCompVector <- function(x, constSumValue = 1.0){
  stopifnot(is.vector(x))
  stopifnot(!is.na(x))
  stopifnot(all(0 < x))
  stopifnot(all(x < constSumValue))
  stopifnot(all.equal(sum(x), constSumValue))
}

#' Check whether X is a valid closed, compositional matrix.
#'
#' @param X Matrix comprised of row vectors, each of which is a closed,
#' compositional vector
#'
#' @param constSumValue Positive-value scalar that is the sum of the
#' elements in the composition
#'
#' @noRd
CheckCompMatrix <- function(X, constSumValue = 1.0){
  stopifnot(is.matrix(X))
  stopifnot(nrow(X) >= 2)
  stopifnot(ncol(X) >= 2)
  apply(X, 1, CheckCompVector, constSumValue)
}

#' Check whether TT is a valid variation matrix
#'
#' @param TT Variation matrix
#' @noRd
CheckVariationMatrix <- function(TT){
  stopifnot(all(TT[upper.tri(TT)] > 0))
  stopifnot(all(TT[lower.tri(TT)] > 0))
  stopifnot(all.equal(unname(diag(TT)), rep.int(0, ncol(TT))))
  stopifnot(all(TT == t(TT)))
}

#' Check whether Psi is a valid contrast matrix
#'
#' @param Psi Contrast matrix
#'
#' @noRd
CheckPsiMatrix <- function(Psi){
  D <- ncol(Psi)
  stopifnot(nrow(Psi) + 1 == D)

  # first of equation 4.5, p. 36
  stopifnot(all.equal(Psi %*% t(Psi), diag(D-1)))

  # second of equation 4.5, p. 36
  a <- rep.int(1, D)
  A <- diag(D) - (1/D) * a %o% a
  stopifnot(all.equal(t(Psi) %*% Psi, A))
}

#' Check whether Sigma is a valid covariance matrix
#'
#' @param Sigma Covariance matrix
#'
#' @noRd
CheckCovMatrix <- function(Sigma){
  stopifnot(nrow(Sigma) == ncol(Sigma))
  stopifnot(all.equal(Sigma, t(Sigma)))
  stopifnot(all(eigen(Sigma)$value > 0))
}

#' @title Closure operation
#'
#' @description Apply closure to an unclosed compositional vector.
#'
#' @param x               Unclosed compositional vector.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' Typical values for constSumValue are 1, 100, and 1000000.
#'
#' The closure operation is defined in Definition 2.3, on page 9 of the
#' reference.
#'
#' @return A closed compositional vector is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' Close(c(1, 2, 3), constSumValue = 1.0)
#'
#' @export
Close <- function(x, constSumValue = 1.0){
  CheckConstSumValue(constSumValue)
  CheckCompVector_Unclosed(x)
  return(constSumValue * x / sum(x))
}

#' @title Perturbation operation
#'
#' @description Apply perturbation to two, closed compositional vectors.
#'
#' @param x               Closed compositional vector.
#' @param y               Closed compositional vector.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The lengths of x and y must be equal.
#'
#' The perturbation operation is defined in Definition 3.1, on page 24 of the
#' reference.
#'
#' @return A closed compositional vector is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' (y <- Close( c(1, 2, 1), constSumValue))
#' Perturb(x, y, constSumValue)
#'
#' @export
Perturb <- function(x, y, constSumValue = 1.0){
  CheckConstSumValue(constSumValue)
  CheckCompVector(x, constSumValue)
  CheckCompVector(y, constSumValue)
  stopifnot(length(x) == length(y))
  return(Close(x * y, constSumValue))
}

#' @title Powering operation
#'
#' @description Apply powering to a closed compositional vector.
#'
#' @param x               Closed compositional vector.
#' @param alpha           Scalar.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The perturbation operation is defined in Definition 3.2, on page 24 of the
#' reference.
#'
#' @return A closed compositional vector is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (alpha <- 0.5)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' Power(x, alpha, constSumValue)
#'
#' @export
Power <- function(x, alpha, constSumValue = 1.0){
  CheckConstSumValue(constSumValue)
  CheckCompVector(x, constSumValue)
  return(Close(x^alpha, constSumValue))
}

#' @title Calculate the sample center
#'
#' @description Calculate the sample center of a compositional matrix.
#'
#' @param X               Closed compositional matrix.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The sample center is defined in Definition 5.1, on page 66 of the
#' reference.
#'
#' @return A closed compositional vector is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' (y <- Close( c(1, 2, 1), constSumValue))
#' CalcSampleCenter(rbind(x, y), constSumValue)
#'
#' @export
CalcSampleCenter <- function(X, constSumValue = 1.0){
  CheckConstSumValue(constSumValue)
  CheckCompMatrix(X, constSumValue)
  h <- exp(colMeans(log(X)))
  names(h) <- colnames(X)
  return(Close(h, constSumValue))
}

#' @title Calculate the variation matrix
#'
#' @description Calculate the variation matrix of a compositional matrix.
#'
#' @param X               Closed compositional matrix.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The variation matrix is defined in Definition 5.2, on page 66 of the
#' reference.
#'
#' @return A variation matrix is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' (y <- Close( c(1, 2, 1), constSumValue))
#' (z <- Close( c(2, 1, 4), constSumValue))
#' CalcVariationMatrix(rbind(x, y, z), constSumValue)
#'
#' @export
CalcVariationMatrix <- function(X, constSumValue = 1.0){
  CheckConstSumValue(constSumValue)
  CheckCompMatrix(X, constSumValue)
  D <- ncol(X)
  TT <- matrix(0, nrow=D, ncol=D)
  colnames(TT) <- colnames(X)
  rownames(TT) <- colnames(X)
  for(i in 1:(D-1)){
    for( j in (i+1):D ){
      TT[i, j] <- stats::var(log(X[, i] / X[, j]))
      TT[j, i] <- TT[i, j]
    }
  }
  # Function var computes the variance with 1/(n-1)
  # If you want to compute the variance with 1/n, uncomment the following code:
  # n <- nrow(X)
  # TT <- (n - 1) / n * TT
  return( TT )
}

#' @title Calculate the sample total variance
#'
#' @description Calculate the sample total variance.
#'
#' @param TT              Variation matrix.
#'
#' @details
#' The variation matrix is defined in Definition 5.3, on page 67 of the
#' reference.
#'
#' @return The sample total variance (a scalar) is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' (y <- Close( c(1, 2, 1), constSumValue))
#' (z <- Close( c(2, 1, 4), constSumValue))
#' (TT <- CalcVariationMatrix(rbind(x, y, z), constSumValue))
#' CalcTotalVariance(TT)
#'
#' @export
CalcTotalVariance <- function(TT){
  CheckVariationMatrix(TT)
  return(sum(TT) / (2.0 * ncol(TT)))
}


#' @title Calculate the contrast matrix using the Gram-Schmidt procedure
#'
#' @description Calculate the contrast matrix using the Gram-Schmidt procedure.
#'
#' @param D               Dimension of the simplex.
#'
#' @details
#' The contrast matrix is defined in Definition 4.3, on page 36 of the
#' reference.
#'
#' The algorithm is presented on pages 39-40 of the reference.
#'
#' @return A contrast matrix is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' CalcContrastMatrix(3)
#'
#' @export
CalcContrastMatrix <- function(D){
  stopifnot(D >= 2)
  psi <- matrix( 0, ncol=D, nrow=(D-1) )
  for( i in 1:(D-1) ){
    for( j in 1:(D-i) ){
      tmp <- ( D - i ) * ( D - i + 1 )
      psi[i,j] <- sqrt( 1.0 / tmp )
    }
    psi[i,D-i+1] <- - sqrt( ( D-i ) / (D-i+1) )
  }
  return(psi)
}

#' @title Calculate the contrast matrix using a sign matrix
#'
#' @description Calculate the contrast matrix using a sign matrix, which
#' encodes are sequential binary partion (SBP).
#'
#' @param S               Sign matrix.
#'
#' @details
#' The contrast matrix is defined in Definition 4.3, on page 36 of the
#' reference.
#'
#' The algorithm is presented on pages 38-39 of the reference.
#'
#' @return A contrast matrix is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' S <- rbind(c(1, 1, -1, -1, 1, 1),
#'            c(1, -1, 0, 0, -1, -1),
#'            c(0, 1, 0, 0, -1, -1),
#'            c(0, 0, 0, 0, 1, -1),
#'            c(0, 0, 1, -1, 0, 0))
#' CalcContrastMatrix_SBP(S)
#'
#' @export
CalcContrastMatrix_SBP <- function(S){
  # Add a test of the sign matrix
  # All elements are -1, 0, or 1
  # The number of rows + 1 equals the number of columns

  Psi <- matrix(0, nrow=nrow(S), ncol=ncol(S))
  for( i in 1:nrow(S) ){
    isPlusOne <- (S[i, ] == 1)
    isNegOne <- (S[i, ] == -1)
    r <- sum(isPlusOne)
    s <- sum(isNegOne)

    # eqn. 4.10
    tmp <- sqrt(r * s / (r + s))
    Psi[i, isPlusOne] <- tmp / r
    Psi[i, isNegOne] <- - tmp / s
  }
  return(Psi)
}



#' @title Ilr transformation
#'
#' @description Transform a compositional vector to a vector of isometric
#' log-ratio (ilr) coordinates.
#'
#' @param x               Closed, compositional vector.
#' @param Psi             Contrast matrix.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The ilr transformation is defined in Equation 4.7, on page 37 of the
#' reference.
#'
#' @return A vector comprising the ilr coordinates is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' (Psi <- CalcContrastMatrix(3))
#' Ilr(x, Psi, constSumValue)
#'
#' @export
Ilr <- function(x, Psi, constSumValue = 1.0){
  CheckConstSumValue(constSumValue)
  CheckCompVector(x, constSumValue)
  CheckPsiMatrix(Psi)
  stopifnot(length(x) == ncol(Psi))
  return(as.vector(log(x) %*% t(Psi)))
}

#' @title Inverse ilr transformation
#'
#' @description Transform a vector of isometric
#' log-ratio (ilr) coordinates to a closed compositional vector.
#'
#' @param z               Vector of ilr coordinates.
#' @param Psi             Contrast matrix.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The ilr transformation is defined in Equation 4.8, on page 37 of the
#' reference.
#'
#' @return A closed compositional vector is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' (Psi <- CalcContrastMatrix(3))
#' (z <- Ilr(x, Psi, constSumValue))
#' InvIlr(z, Psi, constSumValue)
#'
#' @export
InvIlr <- function(z, Psi, constSumValue = 1.0){
  CheckConstSumValue(constSumValue)
  CheckPsiMatrix(Psi)
  stopifnot(length(z) == nrow(Psi))
  return(Close(as.vector(exp(z %*% Psi)), constSumValue))
}

#' @title Clr transformation
#'
#' @description Transform a compositional vector to a vector of centered
#' log-ratio (clr) coordinates.
#'
#' @param x               Closed, compositional vector.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The clr transformation is defined on pages 35-35 of the reference. The
#' computational procedure is specified in a cheat-sheet written by
#' Raimon Tolosana-Delgado; the cheat-sheet is not formally published.
#'
#' @return A vector comprising the clr coordinates is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' Clr(x, constSumValue)
#'
#' @export
Clr <- function(x, constSumValue = 1.0){
  CheckConstSumValue(constSumValue)
  CheckCompVector(x, constSumValue)
  D <- length(x)
  clrMatrix <- (1 / D) * (matrix( -1, nrow=D, ncol=D) + D * diag(D))
  z <- as.vector(log(x) %*% clrMatrix)
  names(z) <- names(x)
  return(z)
}

#' @title Inverse clr transformation
#'
#' @description Transform a vector of centered
#' log-ratio (clr) coordinates to a closed compositional vector.
#'
#' @param z               Vector of clr coordinates.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The inverse clr transformation is defined in Equation 4.4, on page 34 of the
#' reference.
#'
#' @return A closed compositional vector is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' (z <- Clr(x, constSumValue))
#' InvClr(z, constSumValue)
#'
#' @export
InvClr <- function(z, constSumValue = 1.0){
  CheckConstSumValue(constSumValue)
  return(Close(exp(z), constSumValue))
}


#' @title Calculate the Aitchison inner product
#'
#' @description Calculate the Aitchison inner product between two, closed
#' compositional vectors.
#'
#' @param x               Closed compositional vector.
#' @param y               Closed compositional vector.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The lengths of x and y must be equal.
#'
#' The inner production is defined in Definition 3.5, on page 26 of the
#' reference.
#'
#' @return A scalar (the inner product) is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' (y <- Close( c(1, 2, 1), constSumValue))
#' CalcInnerProduct(x, y, constSumValue)
#'
#' @export
CalcInnerProduct <- function(x, y, constSumValue = 1.0) {
  CheckConstSumValue(constSumValue)
  CheckCompVector(x, constSumValue)
  CheckCompVector(y, constSumValue)
  stopifnot(length(x) == length(y))

  D <- length(x)
  theSum <- 0.0
  for(i in 1:D ){
    for(j in 1:D ){
      theSum <- theSum + log(x[i] / x[j]) * log(y[i] / y[j])
    }
  }
  return(theSum / (2.0 * D))
}


#' @title Calculate the Aitchison norm
#'
#' @description Calculate the Aitchison norm of a closed
#' compositional vector.
#'
#' @param x               Closed compositional vector.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The norm is defined in Definition 3.6, on page 26 of the
#' reference.
#'
#' @return A scalar (the norm) is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' CalcNorm(x, constSumValue)
#'
#' @export
CalcNorm <- function(x, constSumValue = 1.0){
    return(sqrt(CalcInnerProduct(x, x, constSumValue)))
}

#' @title Calculate the Aitchison distance
#'
#' @description Calculate the Aitchison distance between two, closed
#' compositional vectors.
#'
#' @param x               Closed compositional vector.
#' @param y               Closed compositional vector.
#' @param constSumValue   Sum of the composition.
#'
#' @details
#' The lengths of x and y must be equal.
#'
#' The Aitchison distance is defined in Definition 3.7, on page 26 of the
#' reference.
#'
#' @return A scalar (the distance) is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @examples
#' (constSumValue <- 1.0)
#' (x <- Close( c(1, 2, 3), constSumValue))
#' (y <- Close( c(1, 2, 1), constSumValue))
#' CalcDistance(x, y, constSumValue)
#'
#' @export
CalcDistance <- function(x, y, constSumValue = 1.0){
  w <- Close(y^(-1), constSumValue)
  z <- Perturb(x, w, constSumValue)
  return(CalcNorm(z, constSumValue))
}

#' @title Ilr transformation using a scaled logit
#'
#' @description Frequently compositional data comprises just two parts (that is,
#' the data pertain to a two dimensional simplex),
#' but only one value of the two parts is specified.
#' For example, such data might be a titanium concentration.
#' Although the data could be transformed to isometric log-ratio
#' coordinates using functions CalcContrastMatrix and TransComp2Ilr, it is
#' requires a lot of unnecessary computation and coding. For such cases,
#' the data can be transformed using a scaled logit, which gives exactly the
#' same result.
#'
#' @param x               One value of a two-part composition. Argument x may
#'                        be either a scalar or a vector.
#' @param constSumValue   Sum of the composition.
#'
#' @details The ilr transformation is equation 5 in the reference.
#'
#' @return A scalar (the ilr coordinate) is returned.
#'
#' @references
#' Filzmoser, P., Hron, K., and Reimann, C., 2009, Univariate statistical
#' analysis of enironmental (compositional) data -- Problems and possibilities:
#' Science of the Total Environment, 407, 6100-6108.
#'
#' @examples
#' (constSumValue <- 1.0e6)
#' (x <- c(5.3, 7.2, 4.9))
#' scaledLogit(x, constSumValue)
#'
#' @export
scaledLogit <- function(x, constSumValue = 1){
  return(1/sqrt(2)*log(x/(constSumValue-x)))
}

#' @title Inverse ilr transformation using a scaled logit
#'
#' @description Frequently compositional data comprises just two parts (that is,
#' the data pertain to a two dimensional simplex),
#' but only one value of the two parts is specified.
#' For example, such data might be a titanium concentration.
#'
#' After transformation to isometric log-ratio (ilr)
#' coordinates (using function scaledLogit, for
#' example), the two-part composition has just one ilr coordinate.
#' Although the ilr coordinate could be back transformed to a concentration
#' using functions CalcContrastMatrix and TransIlr2Comp, it is
#' requires a lot of unnecessary computation and coding. For such cases,
#' the ilr coordinate can be back transformed using a inverse scaled logit,
#' which gives exactly the equivalent result. That is, it gives one value of
#' a two-part composition.
#'
#' @param z               An ilr coordinate, corresponding to a two-part
#'                        composition. Argument z may
#'                        be either a scalar or a vector.
#' @param constSumValue   Sum of the composition.
#'
#' @details The inverse ilr transformation is derived from equation 5 in the
#' reference.
#'
#' @return A scalar (the concentration) is returned.
#'
#' @references
#' Filzmoser, P., Hron, K., and Reimann, C., 2009, Univariate statistical
#' analysis of enironmental (compositional) data -- Problems and possibilities:
#' Science of the Total Environment, 407, 6100-6108.
#'
#' @examples
#' (constSumValue <- 1.0e6)
#' (z <- c(-8.589791, -8.373150, -8.645279))
#' invScaledLogit(z, constSumValue)
#'
#' @export
invScaledLogit <- function(z, constSumValue = 1.0){
  return(constSumValue/(1+exp(-sqrt(2)*z)))
}

#' @title Transform a variation matrix to a covariance matrix
#'
#' @description Transform a variation matrix in the simplex to
#' a covariance matrix in isometric log-ratio (ilr) coordinates.
#'
#' @param TT              Variation matrix.
#' @param Psi             Constrast matrix for the ilr transformation.
#'
#' @details
#' The transformation is equation 5.5 on page 77 of the reference.
#'
#' @return The covariance matrix is returned.
#'
#' @references
#' Pawlowsky-Glahn, V., Egozcue, J.J., and Tolosana-Delgado, R., 2015, Modeling
#' and analysis of compositional data: John Wiley and Sons, Ltd.
#'
#' @export
TransVariationMatrix2Cov <- function(TT, Psi) {
  CheckVariationMatrix(TT)
  CheckPsiMatrix(Psi)
  stopifnot(ncol(Psi) == ncol(TT))

  return(-0.5 * Psi %*% TT %*% t(Psi))
}

#' @title Transform a covariance matrix to a variation matrix
#'
#' @description Transform a covariance matrix in isometric log-ratio (ilr)
#' coordinates to a variation matrix in the simplex.
#'
#' @param Sigma           Covariance matrix in ilr coordinates.
#' @param Psi             Constrast matrix for the ilr transformation.
#'
#' @details
#' The equation for the transformation is the last equation in section 2 of
#' the reference.
#'
#' @return The variation matrix is returned.
#'
#' @references
#' Hron, K., Filzmoser, P., 2015, Exploring compositional data with the robust
#' compositional biplot, in Carpita, M., Brentari, E., Qannari, M., eds.,
#' Advances in latent variables - Methods, models and applications,
#' Springer, p 219-226.
#'
#' @export
TransCov2VariationMatrix <- function(Sigma, Psi) {
  CheckCovMatrix(Sigma)
  CheckPsiMatrix(Psi)
  stopifnot(nrow(Psi) == ncol(Sigma))

  X <- t(Psi) %*% Sigma %*% Psi
  D <- ncol(X)
  diagX <- diag( diag(X), D, D )
  J <- matrix(1, nrow=D, ncol=D)
  A <- J %*% diagX
  return(A + t(A) - 2.0 * X)
}






#'
#'
#' # X is a matrix with D columns (each row is a composition)
#' # The component in the last column is the common denominator.
#' #' @export
#'
#' CalcAlrCoefs <- function( X )
#' {
#'     if( ContainsZeros( X ) )
#'         return( 0 )
#'
#'     if( is.matrix( X ) == FALSE )
#'         X <- matrix( X, ncol=length(X) )
#'
#'     D <- ncol( X )
#'     theMatrix <- rbind( diag( D-1 ), rep.int( -1, D-1 ) )
#'
#'     Z <- log( X ) %*% theMatrix
#'
#'     colnames( Z ) <- colnames( X )[-ncol(X)]
#'
#'     return( Z )
#' }
#'
#' # Y must be a matrix
#' #' @export
#'
#' CalcInvAlr <- function( Y, kappa=1 )
#' {
#'     Z <- cbind( Y, rep.int( 0, nrow(Y) ) )
#'     return( Close( exp( Z ), kappa=kappa ) )
#' }
#'
#' # X is a matrix with D columns (each row is a composition)
#' #' @export
#'
#' CalcClrCoefs <- function( X )
#' {
#'     if( ContainsZeros( X ) )
#'         return( 0 )
#'
#'     if( is.matrix( X ) == FALSE )
#'         X <- matrix( X, ncol=length(X) )
#'
#'     D <- ncol( X )
#'     theMatrix <- ( 1 / D ) * ( matrix( -1, nrow=D, ncol=D ) + D * diag( D ) )
#'
#'     Z <- log( X ) %*% theMatrix
#'
#'     colnames( Z ) <- colnames( X )
#'     rownames( Z ) <- rownames( X )
#'
#'     return( Z )
#' }
#'
#' # Y is a matrix
#' #' @export
#'
#' CalcInvClr <- function( Y, kappa=1.0 )
#' {
#'     return( Close( exp( Y ), kappa=kappa ) )
#' }
#'
#'
#' ReverseRowOrder <- function( X )
#' {
#'     N <- nrow(X)
#'     Y <- matrix( 0, ncol=ncol(X), nrow=N )
#'     for( i in 1:N )
#'     {
#'         Y[N-i+1,] <- X[i,]
#'     }
#'     return( Y )
#' }
#'
#' # Matrix Gamma, Defn 4.6, p. 79
#' # covariance matrix of clr-transformed data
#' #' @export
#'
#' CalcGamma <- function( X )
#' {
#'     Z <- CalcClrCoefs( X )
#'     return( cov( Z ) )
#' }
#'
#'
#' # Standardized compostional data
#' #' @export
#'
#' Standardize <- function( X, kappa=1.0 )
#' {
#'     theCentre <- CalcCentre( X, kappa=kappa )
#'     Y <- Perturb( theCentre^(-1), X, kappa=kappa )                            # Y is centred
#'     totalVariance <- CalcTotalVariance( CalcVariationMatrix( Y ) )
#'     Z <- Power( Y, 1/sqrt(totalVariance), kappa=kappa )                       # Z is centred and scaled.
#'
#'     return( list( theCentre=theCentre, totalVariance=totalVariance, standardizedData=Z ) )
#' }
#'
